@@ -32,7 +32,7 @@ BOOL cmd_jobs(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
     int i;
     for(i=0; i<vector_count(gJobs); i++) {
         sObject* job = vector_item(gJobs, i);
-        int size = snprintf(buf, BUFSIZ, "[%d] name: %s pid&pgrp: %d", i+1, SJOB(job).mName, SJOB(job).mPGroup);
+        int size = snprintf(buf, BUFSIZ, "[%d] %s (pgrp: %d)", i+1, SJOB(job).mName, SJOB(job).mPGroup);
         if(!fd_write(nextout, buf, size)) {
             err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
             runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
@@ -82,8 +82,6 @@ BOOL cmd_exit(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
     sCommand* command = runinfo->mCommand;
     
     if(sCommand_option_item(command, "-force")) {
-        xyzsh_kill_all_jobs();
-
         runinfo->mRCode = RCODE_EXIT;
     }
     else {
@@ -415,7 +413,7 @@ BOOL cmd_print(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
     sCommand* command = runinfo->mCommand;
 
     sObject* out;
-    if(sCommand_option_item(command, "-error")) {
+    if(sCommand_option_item(command, "-out-to-error")) {
         out = gStderr;
     }
     else {
