@@ -29,10 +29,10 @@ BOOL cmd_mshow(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
     sCommand* command = runinfo->mCommand;
 
     sObject* object = runinfo->mRecieverObject;
-    hash_it* it = hash_loop_begin(object);
+    uobject_it* it = uobject_loop_begin(object);
     while(it) {
-        sObject* object2 = hash_loop_item(it);
-        char* key = hash_loop_key(it);
+        sObject* object2 = uobject_loop_item(it);
+        char* key = uobject_loop_key(it);
 
         char buf[BUFSIZ];
         char*obj_kind[16] = {
@@ -46,7 +46,7 @@ BOOL cmd_mshow(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
             return FALSE;
         }
 
-        it = hash_loop_next(it);
+        it = uobject_loop_next(it);
     }
 
     runinfo->mRCode = 0;
@@ -64,20 +64,17 @@ static BOOL entry_fun(sObject* nextin, sObject* nextout, sRunInfo* runinfo, sObj
         block = fun_clone_from_stack_block_gc(block, TRUE, NULL); // block is a stack object, so change the memory manager from stack to gc
 
     if(sCommand_option_item(command, "-inherit")) {
-        sObject* parent = hash_item(object, command->mArgsRuntime[1]);
+        sObject* parent = uobject_item(object, command->mArgsRuntime[1]);
         if(parent) {
-            if(SFUN(parent).mParentCount > XYZSH_INHERIT_MAX) {
-                err_msg("can't inherit a parent function because of a parent function number limit", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
-                return FALSE;
+            if(type == T_FUN && (TYPE(parent) == T_FUN || TYPE(parent) == T_NFUN) 
+                || type == T_CLASS && TYPE(parent) == T_CLASS) 
+            {
+                SFUN(block).mParent = parent;
             }
-
-            if(type == T_CLASS && (TYPE(parent) == T_FUN || TYPE(parent) == T_NFUN) || type == T_FUN && TYPE(parent) == T_CLASS) {
+            else {
                 err_msg("can't inherit a parent function beacuse of the different type", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
                 return FALSE;
             }
-
-            SFUN(block).mParent = parent;
-            SFUN(block).mParentCount = SFUN(parent).mParentCount + 1;
         }
         else {
             err_msg("can't inherit. There is not a parent", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
@@ -130,7 +127,7 @@ static BOOL entry_fun(sObject* nextin, sObject* nextout, sRunInfo* runinfo, sObj
         }
     }
 
-    hash_put(object, command->mArgsRuntime[1], block);
+    uobject_put(object, command->mArgsRuntime[1], block);
 
     return TRUE;
 }
@@ -271,6 +268,30 @@ BOOL cmd_class(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
     return TRUE;
 }
 
+BOOL cmd_defined(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
+{
+    sCommand* command = runinfo->mCommand;
+
+    if(command->mArgsNumRuntime > 1) {
+        sObject* object = runinfo->mCurrentObject;
+        int i;
+        for(i=1; i<command->mArgsNumRuntime; i++) {
+            sObject* reciever = runinfo->mCurrentObject;
+            sObject* obj = access_object(command->mArgsRuntime[i], &reciever, runinfo->mRunningObject);
+
+            if(obj) {
+                runinfo->mRCode = 0;
+            }
+            else {
+                runinfo->mRCode = RCODE_NFUN_FALSE;
+                break;
+            }
+        }
+    }
+
+    return TRUE;
+}
+
 BOOL cmd_inherit(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
 {
     sCommand* command = runinfo->mCommand;
@@ -329,7 +350,6 @@ BOOL cmd_inherit(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
 
         case T_NFUN:
         case T_BLOCK:
-        case T_COMPLETION:
             err_msg("There is not a parent object", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
             return FALSE;
 
@@ -465,15 +485,23 @@ BOOL cmd_var(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
             }
 
             int i;
-            int max = vector_size(SFD(nextin).fdbuf.mLines);
+            int max = vector_count(SFD(nextin).fdbuf.mLines);
             for(i=1; i<command->mArgsNumRuntime; i++) {
+                sObject* current = runinfo->mCurrentObject;
+                sObject* tmp = access_object3(command->mArgsRuntime[i], &current);
+
+                if(tmp && sCommand_option_item(command, "-local") && !sCommand_option_item(command, "-force")) {
+                    err_msg("can't overwrite other the same name variable. Use -force option", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
+                    return FALSE;
+                }
+
                 if(i-1 < max) {
                     sObject* new_var = STRING_NEW_GC(vector_item(SFD(nextin).fdbuf.mLines, i-1), TRUE);
                     string_chomp(new_var);
-                    hash_put(object, command->mArgsRuntime[i], new_var);
+                    uobject_put(object, command->mArgsRuntime[i], new_var);
                 }
                 else {
-                    hash_put(object, command->mArgsRuntime[i], STRING_NEW_GC("", TRUE));
+                    uobject_put(object, command->mArgsRuntime[i], STRING_NEW_GC("", TRUE));
                 }
             }
 
@@ -510,7 +538,7 @@ BOOL cmd_object(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
             int i;
             for(i=1; i<command->mArgsNumRuntime; i++) {
                 sObject* new_object = UOBJECT_NEW_GC(8, object, command->mArgsRuntime[i], TRUE);
-                hash_put(object, command->mArgsRuntime[i], new_object);
+                uobject_put(object, command->mArgsRuntime[i], new_object);
                 uobject_init(new_object);
 
                 int rcode = 0;
@@ -527,7 +555,7 @@ BOOL cmd_object(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
             int i;
             for(i=1; i<command->mArgsNumRuntime; i++) {
                 sObject* new_object = UOBJECT_NEW_GC(8, object, command->mArgsRuntime[i], TRUE);
-                hash_put(object, command->mArgsRuntime[i], new_object);
+                uobject_put(object, command->mArgsRuntime[i], new_object);
                 uobject_init(new_object);
                 runinfo->mRCode = 0;
             }
@@ -545,7 +573,7 @@ BOOL cmd_mrun(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
         sObject* klass = class_clone_from_stack_block_stack(command->mBlocks[0], NULL);
 
         int rcode = 0;
-        if(!run(klass, nextin, nextout, &rcode, runinfo->mRecieverObject, runinfo->mRunningObject)) {
+        if(!run(SCLASS(klass).mBlock, nextin, nextout, &rcode, runinfo->mRecieverObject, runinfo->mRunningObject)) {
             runinfo->mRCode = rcode;
             return FALSE;
         }
@@ -614,7 +642,7 @@ BOOL cmd_co(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
     sCommand* command = runinfo->mCommand;
 
     if(command->mArgsNumRuntime == 2) {
-        sObject* new_current = hash_item(gCurrentObject, command->mArgsRuntime[1]);
+        sObject* new_current = uobject_item(gCurrentObject, command->mArgsRuntime[1]);
         if(new_current && TYPE(new_current) == T_UOBJECT) {
             gCurrentObject = new_current;
             runinfo->mRCode = 0;
@@ -689,7 +717,7 @@ BOOL cmd_ref(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
             sObject* object = runinfo->mCurrentObject;
 
             int i;
-            int max = vector_size(SFD(nextin).fdbuf.mLines);
+            int max = vector_count(SFD(nextin).fdbuf.mLines);
             for(i=0; i<max; i++) {
                 char* line = vector_item(SFD(nextin).fdbuf.mLines, i);
                 void* mem = (void*)(unsigned long)strtoll(line, NULL, 16);
@@ -728,7 +756,7 @@ BOOL cmd_ref(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
                 sObject* object = runinfo->mCurrentObject;
 
                 int i;
-                int max = vector_size(SFD(nextin).fdbuf.mLines);
+                int max = vector_count(SFD(nextin).fdbuf.mLines);
                 BOOL shift = sCommand_option_item(command, "-shift");
                 for(i=0; i<max; i++) {
                     if(i+1 < command->mArgsNumRuntime) {
@@ -737,7 +765,7 @@ BOOL cmd_ref(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
                         int type = memchecker_item(gMemChecker, mem);
 
                         if(type >= 0) {
-                            hash_put(object, command->mArgsRuntime[i+1], mem);
+                            uobject_put(object, command->mArgsRuntime[i+1], mem);
                         }
                         else {
                             err_msg("invalid adress. it's not memory managed by a xyzsh gabage collecter", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
@@ -804,7 +832,7 @@ BOOL cmd_ary(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
 
                 if(ary && TYPE(ary) == T_VECTOR) {
                     char buf[128];
-                    int size = snprintf(buf, 128, "%d\n", vector_size(ary));
+                    int size = snprintf(buf, 128, "%d\n", vector_count(ary));
 
                     if(!fd_write(nextout, buf, size)) {
                         err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
@@ -834,14 +862,14 @@ BOOL cmd_ary(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
                     sObject* ary = access_object(command->mArgsRuntime[i], &reciever, runinfo->mRunningObject);
 
                     if(ary && TYPE(ary) == T_VECTOR) {
-                        if(vector_size(ary) > 0) {
+                        if(vector_count(ary) > 0) {
                             int num = atoi(argument);
                             if(num < 0) {
-                                num += vector_size(ary);
+                                num += vector_count(ary);
                                 if(num < 0) num = 0;
                             }
-                            if(num >= vector_size(ary)) {
-                                num = vector_size(ary) -1;
+                            if(num >= vector_count(ary)) {
+                                num = vector_count(ary) -1;
                                 if(num < 0) num = 0;
                             }
 
@@ -875,7 +903,7 @@ BOOL cmd_ary(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
                     sObject* ary = access_object(command->mArgsRuntime[i], &reciever, runinfo->mRunningObject);
                     if(ary && TYPE(ary) == T_VECTOR) {
                         int j;
-                        for(j=0; j<vector_size(ary); j++) {
+                        for(j=0; j<vector_count(ary); j++) {
                             sObject* var = vector_item(ary, j);
 
                             if(!fd_write(nextout, string_c_str(var), string_length(var))) {
@@ -917,12 +945,12 @@ BOOL cmd_ary(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
 
             int append = atoi(argument);
 
-            if(append < 0) append += vector_size(ary) + 1;
+            if(append < 0) append += vector_count(ary) + 1;
             if(append < 0) append = 0;
-            if(append > vector_size(ary)) append = vector_size(ary);
+            if(append > vector_count(ary)) append = vector_count(ary);
 
             int i;
-            int max = vector_size(SFD(nextin).fdbuf.mLines);
+            int max = vector_count(SFD(nextin).fdbuf.mLines);
             void** appended_ary = MALLOC(sizeof(sObject*)*max);
             for(i=0; i<max; i++) {
                 sObject* new_var = STRING_NEW_GC(vector_item(SFD(nextin).fdbuf.mLines, i), FALSE);
@@ -949,9 +977,18 @@ BOOL cmd_ary(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
             }
 
             int i;
-            int max = vector_size(SFD(nextin).fdbuf.mLines);
+            int max = vector_count(SFD(nextin).fdbuf.mLines);
             sObject* new_ary = VECTOR_NEW_GC(16, TRUE);
-            hash_put(object, command->mArgsRuntime[1], new_ary);
+
+            sObject* current = runinfo->mCurrentObject;
+            sObject* tmp = access_object3(command->mArgsRuntime[1], &current);
+
+            if(tmp && sCommand_option_item(command, "-local") && !sCommand_option_item(command, "-force")) {
+                err_msg("can't overwrite other the same name variable. Use -force option", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
+                return FALSE;
+            }
+
+            uobject_put(object, command->mArgsRuntime[1], new_ary);
             for(i=0; i<max; i++) {
                 sObject* new_var = STRING_NEW_GC(vector_item(SFD(nextin).fdbuf.mLines, i), FALSE);
                 string_chomp(new_var);
@@ -1117,7 +1154,7 @@ BOOL cmd_hash(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
             }
 
             int i;
-            int max = vector_size(SFD(nextin).fdbuf.mLines);
+            int max = vector_count(SFD(nextin).fdbuf.mLines);
             sObject* key = STRING_NEW_STACK("");
             for(i=0; i<max; i++) {
                 if((i % 2) == 0) {
@@ -1148,9 +1185,18 @@ BOOL cmd_hash(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
             }
 
             int i;
-            int max = vector_size(SFD(nextin).fdbuf.mLines);
+            int max = vector_count(SFD(nextin).fdbuf.mLines);
             sObject* new_hash = HASH_NEW_GC(16, TRUE);
-            hash_put(object, command->mArgsRuntime[1], new_hash);
+
+            sObject* current = runinfo->mCurrentObject;
+            sObject* tmp = access_object3(command->mArgsRuntime[1], &current);
+
+            if(tmp && sCommand_option_item(command, "-local") && !sCommand_option_item(command, "-force")) {
+                err_msg("can't overwrite other the same name variable. Use -force option", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
+                return FALSE;
+            }
+
+            uobject_put(object, command->mArgsRuntime[1], new_hash);
             sObject* key = STRING_NEW_STACK("");
             for(i=0; i<max; i++) {
                 if((i % 2) == 0) {
@@ -1229,7 +1275,7 @@ BOOL cmd_export(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
 
         if(command->mArgsNumRuntime > 1) {
             int i;
-            int max = vector_size(SFD(nextin).fdbuf.mLines);
+            int max = vector_count(SFD(nextin).fdbuf.mLines);
             for(i=1; i<command->mArgsNumRuntime; i++) {
                 if(i-1 < max) {
                     sObject* new_var = STRING_NEW_STACK(vector_item(SFD(nextin).fdbuf.mLines, i-1));
