@@ -802,11 +802,8 @@ BOOL cmd_readline_insert_text(sObject* nextin, sObject* nextout, sRunInfo* runin
 
     if(runinfo->mFilter) {
         (void)rl_insert_text(SFD(nextin).fdbuf.mBuf);
-#if defined(__FREEBSD__)
-//extern int rl_reset_terminal PARAMS((const char *));
-    mclear_immediately();
-    rl_forced_update_display();
-#endif
+        puts("");
+        rl_forced_update_display();
 
         runinfo->mRCode = 0;
     }
@@ -829,10 +826,8 @@ BOOL cmd_readline_delete_text(sObject* nextin, sObject* nextout, sRunInfo* runin
         if(rl_point < 0) {
             rl_point = 0;
         }
-#if defined(__FREEBSD__)
-    mclear_immediately();
-    rl_forced_update_display();
-#endif
+        puts("");
+        rl_forced_update_display();
 
         runinfo->mRCode = 0;
     }
@@ -912,15 +907,13 @@ static int readline_macro(int count, int key)
     }
 
     rl_insert_text(SFD(nextout2).fdbuf.mBuf);
-#if defined(__FREEBSD__)
-    mclear_immediately();
+    puts("");
     rl_forced_update_display();
-#endif
-
     stack_end_stack();
 
     return 0;
 }
+
 static int skip_quoted(const char *s, int i, char q)
 {
    while(s[i] && s[i]!=q) 
