@@ -764,6 +764,7 @@ BOOL sCommand_put_option_with_argument(sCommand* self, MANAGED char* key, MANAGE
 BOOL sCommand_expand_env(sCommand* command, sObject* object, sObject* nextin, sObject* nextout, sRunInfo* runinfo);
 BOOL sCommand_expand_env_redirect(sCommand* command, sObject* nextin, sRunInfo* runinfo);
 void sCommand_sweep_runtime_info(sCommand* command);
+void sStatment_sweep_runtime_info(sStatment* self);
 
 extern sObject* gMemChecker;
 
@@ -797,6 +798,8 @@ void clear_matching_info_variable();
 #define REDIRECT_IN 1
 #define REDIRECT_APPEND 2
 #define REDIRECT_OUT 3
+#define REDIRECT_ERROR_APPEND 4
+#define REDIRECT_ERROR_OUT 5
 
 BOOL xyzsh_readline_interface_onetime(int* rcode, char* cmdline, int cursor_point, char* source_name, char** argv, int argc, fXyzshJobDone xyzsh_job_done_);
 char* xyzsh_job_title(int n);
